{% extends "layout.html" %}
{% block body %}
<div class="deck">

    <p>
    This document describes the HydrOffice Sound Speed Library and Manager ({{ version }}). For the source code,
    <a href="https://github.com/hydroffice/hyo2_soundspeed">go here</a>.
    </p>

</div>
    {{ body }}
{% endblock %}
