<p class="logo"><a href="{{ pathto(master_doc) }}">
  <img class="logo" src="https://github.com/hydroffice/hyo2_soundspeed/raw/master/hyo2/ssm2/app/gui/soundspeedmanager/media/app_icon.png" alt="Logo"/>
</a></p>
