"""
Hydro-Package
BAG
"""

name = 'BAG'
__version__ = '1.2.12'
__author__ = 'gmasetti@ccom.unh.edu'
__license__ = 'LGPLv3 license'
__copyright__ = 'Copyright (c) 2025, University of New Hampshire, Center for Coastal and Ocean Mapping'
